/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.translation.ui;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import benten.cat.translation.ui.tmengine.ITmEntry;
import benten.cat.translation.ui.tmengine.ITranslationMemory;
import benten.cat.translation.ui.tmengine.TmPreferenceInitializer;

/**
 * The activator class controls the plug-in life cycle
 */
public class TranslationUIPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = TranslationUIPlugin.class.getPackage().getName();

	// The shared instance
	private static TranslationUIPlugin plugin;

	Map<String, ITranslationMemory> tmMap = new HashMap<String, ITranslationMemory>();
	ITranslationMemory tm = null;

	/**
	 * The constructor
	 */
	public TranslationUIPlugin() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static TranslationUIPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path
	 *
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}

	/**
	 * Logs the given message.
	 *
	 * @param message
	 *            the message to log
	 */
	public static void log(String message) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, message));
	}

	/**
	 * Logs the given exception.
	 *
	 * @param ex
	 *            the exception to log
	 */
	public static void log(final Throwable ex) {
		plugin.getLog().log(new Status(IStatus.WARNING, PLUGIN_ID, -1, ex.toString(), ex));
	}

	public void loadLocalTmFromXliff(File fileXliff) {
		tm.loadLocalTmFromXliff(fileXliff);
	}

	public Map<String, ITranslationMemory> getTmMap() {
		return tmMap;
	}

	public void loadExtensions() {

		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint point = registry.getExtensionPoint(PLUGIN_ID + ".engine");
		if (point == null)
			return;

		IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				IConfigurationElement cfgElem = cfgElems[j];
				try {
					String name = cfgElem.getAttribute("name");
					Object helloObj;
					helloObj = cfgElem.createExecutableExtension("class");
					if (helloObj instanceof ITranslationMemory) {
						tmMap.put(name, (ITranslationMemory) helloObj);
					}
				} catch (CoreException e) {
					log(e);
				}
			}

		}

		IPreferenceStore store = TranslationUIPlugin.getDefault().getPreferenceStore();
		String tm_name = store.getString(TmPreferenceInitializer.TM_NAME);
		ITranslationMemory tm = tmMap.get(tm_name);
		if (tm != null) {
			tm.load();
			this.tm = tm;
		}

	}

	public List<ITmEntry> fuzzyMatch(String string) {
		synchronized (this) {
			if (tm == null) {
				log("fuzzyMatch:TMエンジンがロードされていません。");
				return new ArrayList<ITmEntry>();
			}
			return tm.fuzzyMatch(string);
		}
	}

	public List<ITmEntry> getNearList() {
		if (tm == null) {
			log("getNearList:TMエンジンがロードされていません。");
			MessageDialog.openInformation(TranslationUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow()
					.getShell(), "", "getNearList:TMエンジンがロードされていません。");
			return new ArrayList<ITmEntry>();
		}

		return tm.getNearList();
	}

	public void fuzzyMatchLocalTm(String string) {
		if (tm == null) {
			log("fuzzyMatchLocalTm:TMエンジンがロードされていません。");
			MessageDialog.openInformation(TranslationUIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow()
					.getShell(), "", "getNearList:TMエンジンがロードされていません。");
			return;
		}

		tm.fuzzyMatchLocalTm(string);
	}
}
