/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.translation.ui.actions;

import java.io.File;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.PartInitException;

import benten.cat.translation.ui.TranslationUIPlugin;
import benten.cat.translation.ui.views.TranslationView;

public class ShowLocalTranslationAction implements IEditorActionDelegate {

	private IEditorPart targetEditor;
	private ISelection selection;

	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = selection;
	}

	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		this.targetEditor = targetEditor;
	}

	public void run(IAction action) {
		final ITextSelection textSelection = (ITextSelection) selection;
		loadLocalTm(textSelection.getText());
	}

	public void loadLocalTm(final String string) {

		IEditorInput editorInput = targetEditor.getEditorInput();
		IPathEditorInput fileInput = (IPathEditorInput) editorInput;
		IPath path = fileInput.getPath();
		File fileXliff = path.toFile();

		TranslationUIPlugin.getDefault().loadLocalTmFromXliff(fileXliff);

		TranslationView view = null;
		try {
			view = (TranslationView) targetEditor.getSite().getPage().showView(TranslationView.class.getName());
		} catch (PartInitException e) {
			TranslationUIPlugin.log(e);
			return;
		}

		try {
			targetEditor.getSite().getWorkbenchWindow().getWorkbench().getProgressService().run(true, true,
					new IRunnableWithProgress() {
						public void run(IProgressMonitor monitor) throws InvocationTargetException,
								InterruptedException {
							monitor.beginTask("Translating...", 1);
							TranslationUIPlugin.getDefault().fuzzyMatchLocalTm(string);
							monitor.done();
						}
					});
		} catch (InvocationTargetException e) {
			TranslationUIPlugin.log(e);
		} catch (InterruptedException e) {
			TranslationUIPlugin.log(e);
		}
		view.getViewer().setInput(TranslationUIPlugin.getDefault().getNearList());
	}
}
