/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.translation.ui.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import benten.cat.translation.ui.TranslationUIPlugin;
import benten.cat.translation.ui.actions.CopyTranslatinAction;
import benten.cat.translation.ui.tmengine.ITmEntry;

public class TranslationView extends ViewPart {

	private TableViewer viewer;

	private TableColumn tmName;
	private TableColumn source;
	private TableColumn translation;
	private TableColumn matchQuality;
	private Clipboard clipboard;
	private Action copyAction;

	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {

		public String getColumnText(Object obj, int index) {
			ITmEntry entry = (ITmEntry) obj;
			switch (index) {
			case 0:
				return entry.getTMName();
			case 1:
				return entry.getScore();
			case 2:
				return entry.getSrcText();
			case 3:
				return entry.getTranslation();
			}
			return getText(obj);
		}

		public Image getColumnImage(Object obj, int index) {
			return null;
		}
	}

	class NameSorter extends ViewerSorter {
	}

	/**
	 * コンストラクター。
	 */
	public TranslationView() {
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {

		viewer = new TableViewer(parent, SWT.MULTI | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		tmName = new TableColumn(table, SWT.RIGHT);
		tmName.setText("TM");
		tmName.setWidth(80);

		matchQuality = new TableColumn(table, SWT.RIGHT);
		matchQuality.setText("品質");
		matchQuality.setWidth(50);

		source = new TableColumn(table, SWT.LEFT);
		source.setText("ソース");
		source.setWidth(200);

		translation = new TableColumn(table, SWT.LEFT);
		translation.setText("ローカル");
		translation.setWidth(200);

		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(getItems());

		makeActions();
		hookContextMenu();
		//hookDoubleClickAction();
		//contributeToActionBars();
	}

	private List<ITmEntry> getItems() {
		return new ArrayList<ITmEntry>();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				TranslationView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void fillContextMenu(IMenuManager manager) {
		boolean isEmpty = viewer.getSelection().isEmpty();
		copyAction.setEnabled(!isEmpty);
		manager.add(copyAction);

		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public ISelection getSelection() {
		return viewer.getSelection();
	}

	public void showTranslation(String string) {
		TranslationUIPlugin plugin = TranslationUIPlugin.getDefault();
		viewer.setInput(plugin.fuzzyMatch(string));
	}

	public TableViewer getViewer() {
		return viewer;
	}

	public Clipboard getClipBoard() {
		if (clipboard == null) {
			clipboard = new Clipboard(getSite().getShell().getDisplay());
		}

		return clipboard;
	}

	@Override
	public void dispose() {
		if (clipboard != null) {
			clipboard.dispose();
		}
		super.dispose();
	}

	private void makeActions() {

		/*
		action1 = new Action() {
			@Override
			public void run() {
				showMessage("Action 1 executed");
			}
		};
		action1.setText("Action 1");
		action1.setToolTipText("Action 1 tooltip");
		action1.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
				ISharedImages.IMG_OBJS_INFO_TSK));
		*/

		copyAction = new CopyTranslatinAction(this, "コピー");

		/*
		doubleClickAction = new Action() {
			@Override
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection) selection).getFirstElement();
				showMessage("Double-click detected on " + obj.toString());
			}
		};
		*/
	}

	/******
	private Action action1;
	private Action doubleClickAction;

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(action1);
		manager.add(new Separator());
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(action1);
	}

	private void showMessage(String message) {
		MessageDialog.openInformation(viewer.getControl().getShell(), "サンプル・ビュー", message);
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}
	*****/
}
